@@ -7,7 +7,7 @@ package and can be called as pyhull.qconvex, pyhull.qdelauany, etc.
 """
 
 __author__ = "Shyue Ping Ong"
-__version__ = "1.3.9"
+__version__ = "1.4.0"
 __date__ = "Nov 20 2012"
 __maintainer__ = "Shyue Ping Ong"
 __email__ = "shyuep@gmail.com"
@@ -106,3 +106,33 @@ def qvoronoi(options, points):
         '3 4 0 3', '4 4 2 1 3']
     """
     return qhull_cmd("qvoronoi", options, points)
+
+def qhalf(options, halfspaces, interior_point):
+    """
+    Similar to qvoronoi command in command-line qhull.
+
+    Args:
+        option:
+            An options string. Up to two options separated by spaces
+            are supported. See Qhull's qhalf help for info. Typically
+            used options are:
+            Fp
+        halfspaces:
+            List of Halfspaces as input.
+        interior_point:
+            An interior point (see qhalf documentation)
+
+    Returns:
+        Output as a list of strings.
+        E.g., ['3', '4', '     1      1         0 ', '     1     -1      2 ',
+        '    -1      1      2 ', '     1      1      2 ']
+    """
+    points = [list(h.normal) + [h.offset] for h in halfspaces]
+    input = [[len(interior_point), 1]]
+    input.append(interior_point)
+    input.append([len(points[0])])
+    input.append([len(points)])
+    input.extend(points)
+    prep_str = [" ".join(map(str, line)) for line in input]
+    output = getattr(hull, "qhalf")(options, "\n".join(prep_str))
+    return map(string.strip, output.strip().split("\n"))
